/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Initialisation des variables
 * 
 */

/* Canvas */
var canvas; // l'objet canvas de la page html
var stage; // la scène graphique de l'application, ajout d'enfant graphique dedans


// les graphismes
var backgroundGame; // le background


//[Title View]
var backgroundMain; // le Background principal
var btnStart; // bouton start dans le menu
var btnCredits; // bouton credits dans le menu


//[Credits]
var credits; // l'écran de credits


//[Game View]
var player; // le joueur
var ball; // la balle
var cpu; // l'ordinateur
var win; // popup gagné
var lose; // popup perdu


// mouvement du joueur avec un effet
var velY = 0,
    speed = 2.3,// vitesse de déplacement
    friction = 0.98;
var keys = [];// tableau qui contient les touches du clavier pressées


//[Score]
var playerScore; // le score du joueur
var cpuScore; // le score de l'ordinateur
var cpuSpeed = 8.5; // la vitesse de l'ordinateur, plus il est rapide, plus le jeu est difficile


// les Variables du jeu
var xSpeed = 10;
var ySpeed = 10;
var tkr = new Object;


//preloader
var preloader;
var loaderWidth;
var bar; // barre de progression du chargement
var manifest;


//[Title View Group]
var TitleView = new Container();


console.log("Jeu du Pong");

document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
	console.log("Main");

	/* récupère le Canvas html */
	canvas = document.getElementById('PongStage');
  	stage = new Stage(canvas);	

  	stage.mouseEventsEnabled = true;// écoute la souris

  	
	/* Utilise le plugin Flash Player pour les navigateurs qui ne supportent pas le son javascript */
  	SoundJS.FlashPlugin.BASE_PATH = "assets/";
    
    if (!SoundJS.checkPlugin(true))
    {
		alert("Erreur de chargement du plugin flash !");
		return;
    }

    // ajoute un identifiant pour les éléments chargés pendant le jeu
  	manifest = [
				{src:"include/images/bg.png", id:"backgroundGame"},
				{src:"include/images/main.png", id:"backgroundMain"},
				{src:"include/images/startB.png", id:"btnStart"},
				{src:"include/images/creditsB.png", id:"btnCredits"},
				{src:"include/images/credits.png", id:"credits"},
				{src:"include/images/paddle.png", id:"cpu"},
				{src:"include/images/paddle.png", id:"player"},
				{src:"include/images/ball.png", id:"ball"},
				{src:"include/images/win.png", id:"win"},
				{src:"include/images/lose.png", id:"lose"},
				{src:"include/sound/playerScore.mp3|include/sound/playerScore.ogg", id:"playerScore"},
				{src:"include/sound/enemyScore.mp3|include/sound/enemyScore.ogg", id:"enemyScore"},
				{src:"include/sound/hit.mp3|include/sound/hit.ogg", id:"hit"},
				{src:"include/sound/wall.mp3|include/sound/wall.ogg", id:"wall"}
			];

	// création de la barre de chargement
	var barHeight = 20;
	loaderColor = createjs.Graphics.getRGB(0,111,119);// couleur #006F77
	loaderBar = new createjs.Container();

	bar = new createjs.Shape();
	bar.graphics.beginFill(loaderColor).drawRect(0, 0, 1, barHeight).endFill();

	loaderWidth = 300;// la taille de la barre de chargement

	// le fond de la barre de progression
	var bgBar = new createjs.Shape();
	var padding = 3
	bgBar.graphics.setStrokeStyle(1).beginStroke(loaderColor).drawRect(-padding/2, -padding/2, loaderWidth+padding, barHeight+padding);

	loaderBar.x = canvas.width - loaderWidth>>1;
	loaderBar.y = canvas.height - barHeight>>1; 
	loaderBar.addChild(bar, bgBar);    

	stage.addChild(loaderBar);

	// chargement et initialisation des éléments		
  	preloader = new PreloadJS();
  	preloader.installPlugin(SoundJS);

  	// écouteurs 
    preloader.onProgress = handleProgress;
    preloader.onComplete = handleComplete;
    preloader.onFileLoad = handleFileLoad;

    // chargement des éléments du jeu
    preloader.loadManifest(manifest);

	/* Ticker */
	Ticker.setFPS(30);
	Ticker.addListener(stage);
}


/**
 * Progession du chargement du jeu
 * 
 * @param  {[type]} event [description]
 * @return {[type]}       [description]
 */
function handleProgress(event)
{
	bar.scaleX = event.loaded * loaderWidth;// le pourcentage de chargement total de l'ensemble des fichiers
}

/**
 * fin du chargement des éléments du jeu
 * @param  {[type]} event [description]
 * @return {[type]}       [description]
 */
function handleComplete(event)
{
	//loaderBar.visible = false;// masque la barre de progression
	stage.removeChild(loaderBar);// supprime la barre de progression

	addTitleView();// ajoute les éléments graphique du jeu à la fin du chargement de tous les éléments
}


/**
 * Fichier chargé, un seul élément de la liste est chargé,
 * chargement un élément par un.
 * 
 * @param  {[type]} event [description]
 * @return {[type]}       [description]
 */
function handleFileLoad(event)
{
     
	switch(event.type)
	{
		case PreloadJS.IMAGE:
			//image loaded
			var img = new Image();
			img.src = event.src;
			window[event.id] = new Bitmap(img);
			break;

		/*case PreloadJS.SOUND:
			//sound loaded
			break;*/
	}
}


/**
 * Création de l'interface Accueil
 * 
 */
function addTitleView()
{
	console.log("Add Title View");

	// positionne les éléments
	btnStart.x = 240 - 32;
	btnStart.y = 160;
	btnStart.name = 'btnStart';
	
	btnCredits.x = btnStart.x;
	btnCredits.y = 200;
	
	// ajoute sur l'interface de titre, les boutons
	TitleView.addChild(backgroundMain, btnStart, btnCredits);

	// ajout le background et l'interface titre
	stage.addChild(backgroundGame, TitleView);
	stage.update();
	
	// écouteur sur les boutons
	btnStart.onPress = tweenTitleView;
	btnCredits.onPress = showCredits;
}


/**
 * L'interface d'affichage des crédits / auteur
 * 
 * @return {[type]} [description]
 */
function showCredits()
{	
	credits.x = 480;
		
	stage.addChild(credits);
	stage.update();

	Tween.get(credits).to({x:0}, 300);// amene l'interface à la position x=0 px, avec une vitesse de 300ms
	credits.onPress = hideCredits;// masque l'interface sur le clic de la souris
}


/**
 * Masque l'interface crédits
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function hideCredits(e)
{
	// déplace l'interface à x=480 px, à une vitesse de 300 ms, à la fin de la tween, la fonction removeCredits est appelée 
	Tween.get(credits).to({x:480}, 300).call(removeCredits);// creation d'un tween
}



/**
 * Suppression de la vue crédits
 * 
 * @return {[type]} [description]
 */
function removeCredits()
{
	stage.removeChild(credits);
}



/**
 * Effet de Tween pour afficher l'interface du jeu
 * 
 * @return {[type]} [description]
 */
function tweenTitleView()
{		
	// lance le jeu
	Tween.get(TitleView).to({y:-320}, 300).call(addGameView);
}


/**
 * Affiche l'interface du jeu
 * 
 */
function addGameView()
{
	// supprime les autres interfaces
	stage.removeChild(TitleView);
	TitleView = null;
	credits = null;

	
	// création des personnages
	player.x = 2;
	player.y = 160 - 37.5;
	cpu.x = 480 - 25;
	cpu.y = 160 - 37.5;
	ball.x = 240 - 15;
	ball.y = 160 - 15;

	
	// affiche le score
	playerScore = new Text('0', 'bold 28px Arial', '#006F77');
	playerScore.x = 211;
	playerScore.y = 28;
	
	cpuScore = new Text('0', 'bold 28px Arial', '#006F77');
	cpuScore.x = 262;
	cpuScore.y = 28;
	

	// ajout des éléments graphiques
	stage.addChild(playerScore, cpuScore, player, cpu, ball);
	stage.update();
	

	// clic sur l'écran pour lancer le jeu
	backgroundGame.onPress = startGame;
}



/**
 * Le jeu démarre
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function startGame(e)
{
	backgroundGame.onPress = null;
	stage.onMouseMove = movePaddle;// écouteur sur la souris pour déplacer le joueur
	
	document.body.addEventListener("keydown", updateKeyDown );// ecoute le clavier pour déplacer le joueur
	document.body.addEventListener("keyup", updateKeyUp );

	updatePlayer();// déplacement du joueur

	Ticker.addListener(tkr, false);
	tkr.tick = update;
}


/**
 * Mets à jour la touche du clavier appuyée par le joueur
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function updateKeyDown(e)
{
	// si touche flèche haut ou flèche bas
	if( e.keyCode == 38 | e.keyCode == 40)
		keys[e.keyCode] = true;
}



/**
 * Mets à jour la touche du clavier relachée par le joueur
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function updateKeyUp(e)
{
	// si touche flèche haut ou flèche bas
	if( e.keyCode == 38 | e.keyCode == 40)
		keys[e.keyCode] = false;
}


/**
 * Déplace le joueur
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function updatePlayer(e)
{
	// si touche flèche bas
	if (keys[38])
	{
		if (velY > -speed && velY > -10)
			velY--;
		else
			velY = 0;
	}

	// si touche flèche haut
	else if (keys[40])
	{
		if (velY < speed && velY < 10)
			velY++;
		else
			velY = 0;
	}

	// limite la vitesse de déplacement
	if( velY > -10 && velY < 10)
		velY *= friction;
    
    player.y += velY;

    if (player.y > 395)
        player.y = 395;
    else if (player.y <= 0)
        player.y = 0;

    // si la partie continue
    if( parseInt( playerScore.text ) < 5 && parseInt( cpuScore.text ) < 5)
    {
    	setTimeout(updatePlayer, 10);// appel dans 10 ms
    	document.body.removeEventListener("keydown", updateKeyDown );// ecoute le clavier pour déplacer le joueur
		document.body.removeEventListener("keyup", updateKeyUp );
    }	
}



/**
 * Déplace le joueur
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function movePaddle(e)
{
	player.y = e.stageY;// suit la position verticale de la souris
}



/**
 * Un des 2 joueur a raté la balle
 * Remise en jeu
 * 
 * @return {[type]} [description]
 */
function reset()
{
	ball.x = 240 - 15;
	ball.y = 160 - 15;
	player.y = 160 - 37.5;
	cpu.y = 160 - 37.5;
	
	stage.onMouseMove = null;
	Ticker.removeListener(tkr);
	backgroundGame.onPress = startGame;
}


/**
 * Gestion de l'affichage du jeu pendant la partie
 * 
 * Mets à jour le déplacement de la balle
 * et de l'ordinateur
 * 
 * @return {[type]} [description]
 */
function update()
{
	// deplacement de la balle
	ball.x = ball.x + xSpeed;
	ball.y = ball.y + ySpeed;
	

	// deplacement de l'ordinateur
	if((cpu.y+32) < (ball.y-14))
		cpu.y = cpu.y + cpuSpeed;
	else if((cpu.y+32) > (ball.y+14))
		cpu.y = cpu.y - cpuSpeed;
	

	// gestion des collisions entre le mur et la balle
	if((ball.y) < 0)// mur haut
	{
		ySpeed = -ySpeed;
		SoundJS.play('wall');
	}	
	
	else if((ball.y + (30)) > 320)// mur bas
	{
		ySpeed = -ySpeed;
		SoundJS.play('wall');
	}
		

	// score de l'ordinateur a gagne 1 point
	if((ball.x) < 0)
	{
		xSpeed = -xSpeed;
		cpuScore.text = parseInt(cpuScore.text + 1);
		reset();
		SoundJS.play('enemyScore');
	}
	

	// score du joueur a gagne 1 point
	else if((ball.x + (30)) > 480)
	{
		xSpeed = -xSpeed;
		playerScore.text = parseInt(playerScore.text + 1);
		reset();
		SoundJS.play('playerScore');
	}
	

	// l'ordinateur touche la balle
	if(ball.x + 30 > cpu.x && ball.x + 30 < cpu.x + 22 && ball.y >= cpu.y && ball.y < cpu.y + 75)
	{
		xSpeed *= -1;
		SoundJS.play('hit');
	}
	
	// le joueur touche la balle
	if(ball.x <= player.x + 22 && ball.x > player.x && ball.y >= player.y && ball.y < player.y + 75)
	{
		xSpeed *= -1;
		SoundJS.play('hit');
	}
	
	// empeche le joueur de sortir de l'écran
	if(player.y >= 239)
		player.y = 239;

	
	// verifie si le joueur a gagne
	if(playerScore.text == '5')
		endGame('win');
	

	// verifie si le joueur a perdu
	else if(cpuScore.text == '5')
		endGame('lose');

}


/**
 * Interface de fin du jeu
 * 
 * @param  {[type]} e [description]
 * @return {[type]}   [description]
 */
function endGame(e)
{
	Ticker.removeListener(tkr);
	stage.onMouseMove = null;
	backgroundGame.onPress = null
	
	// supprime les graphismes ordinateur et joueur
	stage.removeChild(player);
	stage.removeChild(cpu);


	// si le joueur a gagne
	if(e == 'win')
	{
		win.x = 140;
		win.y = -90;
	
		stage.addChild(win);
		Tween.get(win).to({y: 115}, 300);
	}
	else // sinon perdu
	{
		lose.x = 140;
		lose.y = -90;
	
		stage.addChild(lose);
		Tween.get(lose).to({y: 115}, 300);
	}
}


